package com.sun.security.sasl.ntlm;

import com.sun.security.ntlm.NTLMException;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.io.IOException;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.sasl.RealmCallback;
import javax.security.sasl.SaslException;
import javax.security.auth.callback.CallbackHandler;
import java.util.Map;
import java.util.Random;
import com.sun.security.ntlm.Client;
import javax.security.sasl.SaslClient;

final class NTLMClient implements SaslClient
{
    private static final String NTLM_VERSION = "com.sun.security.sasl.ntlm.version";
    private static final String NTLM_RANDOM = "com.sun.security.sasl.ntlm.random";
    private static final String NTLM_DOMAIN = "com.sun.security.sasl.ntlm.domain";
    private static final String NTLM_HOSTNAME = "com.sun.security.sasl.ntlm.hostname";
    private final Client client;
    private final String mech;
    private final Random random;
    private int step;
    
    NTLMClient(String mech, String defaultName, String s, String defaultRealmInfo, Map<String, ?> map, CallbackHandler callbackHandler) throws SaslException {
        this.step = 0;
        this.mech = mech;
        String property = null;
        Random random = null;
        String canonicalHostName = null;
        if (map != null) {
            String s2 = (String)map.get("javax.security.sasl.qop");
            if (s2 != null && !s2.equals("auth")) {
                throw new SaslException("NTLM only support auth");
            }
            property = (String)map.get("com.sun.security.sasl.ntlm.version");
            random = (Random)map.get("com.sun.security.sasl.ntlm.random");
            canonicalHostName = (String)map.get("com.sun.security.sasl.ntlm.hostname");
        }
        this.random = ((random != null) ? random : new Random());
        if (property == null) {
            property = System.getProperty("ntlm.version");
        }
        Object o = (defaultRealmInfo != null && !defaultRealmInfo.isEmpty()) ? new RealmCallback("Realm: ", defaultRealmInfo) : new RealmCallback("Realm: ");
        Object o2 = (defaultName != null && !defaultName.isEmpty()) ? new NameCallback("User name: ", defaultName) : new NameCallback("User name: ");
        PasswordCallback o3 = new PasswordCallback("Password: ", false);
        try {
            callbackHandler.handle(new Callback[] { o, o2, o3 });
        }
        catch (UnsupportedCallbackException ex) {
            throw new SaslException("NTLM: Cannot perform callback to acquire realm, username or password", ex);
        }
        catch (IOException ex2) {
            throw new SaslException("NTLM: Error acquiring realm, username or password", ex2);
        }
        if (canonicalHostName == null) {
            try {
                canonicalHostName = InetAddress.getLocalHost().getCanonicalHostName();
            }
            catch (UnknownHostException ex3) {
                canonicalHostName = "localhost";
            }
        }
        try {
            String name = o2.getName();
            if (name == null) {
                name = defaultName;
            }
            String text = o.getText();
            if (text == null) {
                text = defaultRealmInfo;
            }
            this.client = new Client(property, canonicalHostName, name, text, o3.getPassword());
        }
        catch (NTLMException ex4) {
            throw new SaslException("NTLM: client creation failure", ex4);
        }
    }
    
    @Override
    public String getMechanismName() {
        return this.mech;
    }
    
    @Override
    public boolean isComplete() {
        return this.step >= 2;
    }
    
    @Override
    public byte[] unwrap(byte[] array, int n, int n2) throws SaslException {
        throw new IllegalStateException("Not supported.");
    }
    
    @Override
    public byte[] wrap(byte[] array, int n, int n2) throws SaslException {
        throw new IllegalStateException("Not supported.");
    }
    
    @Override
    public Object getNegotiatedProperty(String s) {
        if (!this.isComplete()) {
            throw new IllegalStateException("authentication not complete");
        }
        int n = -1;
        switch (s.hashCode()) {
            case -1548608927: {
                if (s.equals("javax.security.sasl.qop")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 11529379: {
                if (s.equals("com.sun.security.sasl.ntlm.domain")) {
                    n = 1;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                return "auth";
            }
            case 1: {
                return this.client.getDomain();
            }
            default: {
                return null;
            }
        }
    }
    
    @Override
    public void dispose() throws SaslException {
        this.client.dispose();
    }
    
    @Override
    public boolean hasInitialResponse() {
        return true;
    }
    
    @Override
    public byte[] evaluateChallenge(byte[] type2) throws SaslException {
        ++this.step;
        if (this.step == 1) {
            return this.client.type1();
        }
        try {
            byte[] array = new byte[8];
            this.random.nextBytes(array);
            return this.client.type3(type2, array);
        }
        catch (NTLMException ex) {
            throw new SaslException("Type3 creation failed", ex);
        }
    }
}
